package com.supervision.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.exception.BusinessException;
import com.supervision.model.AskPatientAnswer;
import com.supervision.model.AskTemplateQuestionLibrary;
import com.supervision.model.DiagnosisQaRecord;
import com.supervision.model.Process;
import com.supervision.pojo.vo.TalkReqVO;
import com.supervision.pojo.vo.TalkResultResVO;
import com.supervision.service.AskPatientAnswerService;
import com.supervision.service.AskTemplateQuestionLibraryService;
import com.supervision.service.ProcessService;
import com.supervision.util.*;
import com.supervision.service.AskService;
import com.supervision.pojo.vo.ActionDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.Random;

@Slf4j
@Service
@RequiredArgsConstructor
public class AskServiceImpl implements AskService {

    private final ProcessService processService;

    private final AskTemplateQuestionLibraryService askTemplateQuestionLibraryService;

    private final AskPatientAnswerService askPatientAnswerService;


    @Override
    public String receiveVoiceFile(MultipartFile file) {
        if (file.getSize() <= 0) {
            throw new BusinessException("语音内容为空");
        }
        // 获取音频对应的文字
        String text = null;
        try {
            text = AsrUtil.asrTransformByBytes(file.getBytes());
        } catch (Exception e) {
            throw new BusinessException("获取语音失败");
        }
        if (StrUtil.isEmpty(text)) {
            throw new BusinessException("语音内容为空");
        }
        return text;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TalkResultResVO talk(TalkReqVO talkReqVO) {
        // 根据processId找到对应的病人
        Process process = Optional.ofNullable(processService.getById(talkReqVO.getProcessId())).orElseThrow(() -> new BusinessException("未找到诊疗进程"));
        // 调用rasa获取文字内容
        String rasaResult = RasaUtil.talkRasa(talkReqVO.getText(), UserUtil.getUser().getId(), process.getPatientId());
        // 如果rasa没有识别出来,则返回默认值
        if (StrUtil.isBlank(rasaResult)) {
            // 这里调用京东数字人接口首先根据token获取房间号
            String roomId = HumanUtil.queryRoomId(talkReqVO.getRoomKey(), talkReqVO.getRoomToken());
            HumanUtil.textDriven("您好,我没有听懂您说什么", roomId);
        }
        TalkResultResVO talkResultResVO = new TalkResultResVO();
        // 这里校验,rasa回复的结果是不是action
        // 这里设置的模板,对于action的动作全部是用ancillary_ | tool_进行标记,详情看生成rasa的yml的代码:RasaServiceImpl.generateDomain
        // ancillary_ | tool_
        if (rasaResult.startsWith("ancillary_") || rasaResult.startsWith("tool_")) {
            List<String> actionList = StrUtil.split(rasaResult, '_');
            if (actionList.size() > 1) {
                ActionDTO actionDTO = new ActionDTO();
                actionDTO.setActionType(actionList.get(0));
                actionDTO.setActionId(actionList.get(1));
                // 在这里给socket回复,设置为动作
                talkResultResVO.setAction(actionDTO);
                talkResultResVO.setType(2);
                return talkResultResVO;
            }
        } else {
            String roomId = HumanUtil.queryRoomId(talkReqVO.getRoomKey(), talkReqVO.getRoomToken());
            AskTemplateQuestionLibrary library = askTemplateQuestionLibraryService.getById(rasaResult);
            if (ObjectUtil.isEmpty(library)) {
                HumanUtil.textDriven("您好,我没有听懂您说什么", roomId);
            } else {
                AskPatientAnswer askPatientAnswer = askPatientAnswerService.lambdaQuery().eq(AskPatientAnswer::getPatientId, process.getPatientId())
                        .eq(AskPatientAnswer::getLibraryQuestionId, library.getId()).last("limit 1").one();
                if (ObjectUtil.isEmpty(askPatientAnswer) || CollUtil.isEmpty(askPatientAnswer.getAnswer())) {
                    HumanUtil.textDriven("您好,我没有听懂您说什么", roomId);
                } else {
                    String resText = askPatientAnswer.getAnswer().get(RandomUtil.randomInt(0, askPatientAnswer.getAnswer().size()));
                    HumanUtil.textDriven(resText, roomId);
                    // 保存记录
                    DiagnosisQaRecord record = new DiagnosisQaRecord();
                    record.setProcessId(talkReqVO.getProcessId());
                    record.setQuestionAnswerId(askPatientAnswer.getId());
                    record.setQuestion(talkReqVO.getText());
                    record.setAnswer(resText);
                    record.setCreateUserId(UserUtil.getUser().getId());
                    record.insert();
                }
            }
        }
        talkResultResVO.setType(1);
        return talkResultResVO;
    }


    @Override
    public String replyVoice() {
        String text = "测试:这是文字转语音的测试,测试是否OK";
        return TtsUtil.ttsTransform(text);
    }


}
